﻿using System;
using NUnit.Framework;
using Vita.Domain.Accounting;

namespace Vita.Domain.Tests.Unit.Accounting
{
    [TestFixture]
    public class CreateAccountTests : TestFixtureBase<Account>
    {
        [SetUp]
        public void SetUp()
        {
            var defaultDate = DateTimeOffset.Parse("2013-01-18T02:07:00+00:00");
            TimeWithOffsetProvider.Current = new TestTimeWithOffsetProvider(defaultDate);

            GuidProvider.Current = new TestGuidProvider();
        }

        [TearDown]
        public void TearDown()
        {
            TimeWithOffsetProvider.ResetDefaults();
            GuidProvider.ResetDefaults();
        }

        [Test]
        public void CreateAccount()
        {
            var accountId = GuidProvider.Current.NewId();
            var openingBalance = new Money((decimal)1.0);
            var accountName = "Current Account";
            var accountHolderId = Guid.NewGuid();
            var accountHolderName = "Dev";
            AccountType accountType = AccountType.Asset;

            Run(
                given: () => new Account(),
                when: account => account.Open(accountId, accountType, accountHolderId, accountHolderName, accountName, openingBalance),
                then: account => new[] {new AccountOpened(account.Id, accountType, accountHolderId, accountHolderName, accountName, openingBalance)}
                );
        }
    }
}