﻿using NUnit.Framework;
using Vita.Domain.Accounting;

namespace Vita.Domain.Tests.Unit.Accounting
{
    [TestFixture]
    public class MoneyTests
    {
        [Test]
        public void PlusOperator_GivenTwoValues_ShouldReturnTotal()
        {
            // Arrange
            var m1 = new Money((decimal)1.1);
            var m2 = new Money((decimal)2.2);

            // Act
            var result = m1 + m2;

            // Assert
            Assert.AreEqual(3.3, result.Value);
            Assert.AreNotSame(result, m1);
            Assert.AreNotSame(result, m2);
        }

        [Test]
        public void MinusOperator_GivenTwoValues_ShouldReturnTotal()
        {
            // Arrange
            var m1 = new Money((decimal)1.1);
            var m2 = new Money((decimal)2.2);

            // Act
            var result = m1 - m2;

            // Assert
            Assert.AreEqual(-1.1, result.Value);
            Assert.AreNotSame(result, m1);
            Assert.AreNotSame(result, m2);
        }

        [Test]
        public void Equals_GivenTwoMoneys_ShouldReturnTrueIfValueIsTheSame()
        {
            // Arrange
            var m1 = new Money((decimal)1.1);
            var m2 = new Money((decimal)1.1);

            // Act
            bool result = m1.Equals(m2);

            // Assert
            Assert.IsTrue(result);
        }

        [Test]
        public void Invert_GivenPositiveAmount_ShouldReturnAdditiveInverseOfSameCurrency()
        {
            // Arrange
            var m1 = new Money((decimal)1.1);
            var expectedResult = new Money(0 - m1.Value);

            // Act
            var result = m1.Invert();

            // Assert
            Assert.AreEqual(expectedResult, result);
        }
    }
}