﻿using NUnit.Framework;
using Vita.Domain.Accounting;

namespace Vita.Domain.Tests.Unit.Accounting
{
    [TestFixture]
    public class TransferTest : TestFixtureBase<FundsTransfer>
    {
        [Test]
        public void RequestFundsTransfer()
        {
            var sourceAccountId = NewId();
            var destinationAccountId = NewId();
            var amount = new Money((decimal)10.0);

            Run(
                given: () => new FundsTransfer(),
                when: transfer => transfer.RequestFundsTransfer(sourceAccountId, destinationAccountId, amount),
                then: transfer => new[] { new FundsTransferRequested(transfer.Id, sourceAccountId, destinationAccountId, amount) });
        }

        [Test]
        public void CannotTransferToTheSameAccount()
        {
            var sourceAccountId = NewId();
            var destinationAccountId = sourceAccountId;
            var amount = new Money((decimal)10.0);

            Run(
                given: () => new FundsTransfer(),
                when: transfer => transfer.RequestFundsTransfer(sourceAccountId, destinationAccountId, amount),
                expectException: typeof(DomainException));
        }

        [Test]
        public void CannotTransferNegativeAmount()
        {
            var sourceAccountId = NewId();
            var destinationAccountId = NewId();
            var amount = new Money((decimal)-10.0);

            Run(
                given: () => new FundsTransfer(), 
                when: transfer => transfer.RequestFundsTransfer(sourceAccountId, destinationAccountId, amount),
                expectException: typeof(DomainException));
        }

        [Test]
        public void CannotTransferZeroAmount()
        {
            var sourceAccountId = NewId();
            var destinationAccountId = NewId();
            var amount = new Money((decimal)0.0);

            Run(
                given: () => new FundsTransfer(),
                when: transfer => transfer.RequestFundsTransfer(sourceAccountId, destinationAccountId, amount),
                expectException: typeof(DomainException));
        }
    }
}