﻿using System;
using System.Collections.Generic;
using System.Linq;
using NUnit.Framework;
using Regalo.Core;
using Vita.Domain.Accounting;

namespace Vita.Domain.Tests.Unit
{
    public abstract class TestFixtureBase<TAggregateRoot>
        where TAggregateRoot : AggregateRoot
    {
        private int _nextId = 0;

        public void Run(Func<TAggregateRoot> given, Action<TAggregateRoot> when, Func<TAggregateRoot, IEnumerable<Event>> then)
        {
            var aggregateRoot = given();
            aggregateRoot.AcceptUncommittedEvents();

            when(aggregateRoot);

            var expected = then(aggregateRoot).ToArray();
            var actual = aggregateRoot.GetUncommittedEvents().ToArray();

            for (int i = 0; i < expected.Length; i++)
            {
                Assert.AreEqual(expected[i], actual[i], string.Format("Expected event at position {0} with text: \"{1}\"", i, expected));
            }
        }

        protected void Run(Func<TAggregateRoot> given, Action<TAggregateRoot> when, Type expectException)
        {
            var aggregateRoot = given();
            aggregateRoot.AcceptUncommittedEvents();

            Assert.Throws(expectException, () => when(aggregateRoot));
        }

        protected Guid NewId()
        {
            return Guid.Parse("00000000-0000-0000-0000-" + (++_nextId).ToString("d12"));
        }
    }
}