using System;
using Regalo.Core;

namespace Vita.Domain.Accounting
{
    public class AccountDebited : Event
    {
        public Guid AggregateId { get; private set; }
        public DateTimeOffset Date { get; private set; }
        public string Payee { get; private set; }
        public Money Amount { get; private set; }
        public string PaymentMethod { get; private set; }
        public string Notes { get; private set; }
        public string[] Categories { get; private set; }
        public string[] Tags { get; private set; }

        public AccountDebited(Guid aggregateId, DateTimeOffset date, string payee, Money amount, string paymentMethod, string notes, string[] categories, string[] tags)
        {
            AggregateId = aggregateId;
            Date = date;
            Payee = payee;
            Amount = amount;
            PaymentMethod = paymentMethod;
            Notes = notes;
            Categories = categories;
            Tags = tags;
        }

        public bool Equals(AccountDebited other)
        {
            if (ReferenceEquals(null, other)) return false;
            if (ReferenceEquals(this, other)) return true;
            return other.AggregateId.Equals(AggregateId) && other.Date.Equals(Date) && Equals(other.Payee, Payee) && Equals(other.Amount, Amount) && Equals(other.PaymentMethod, PaymentMethod) && Equals(other.Notes, Notes) && Equals(other.Categories, Categories) && Equals(other.Tags, Tags);
        }

        public override bool Equals(object obj)
        {
            if (ReferenceEquals(null, obj)) return false;
            if (ReferenceEquals(this, obj)) return true;
            if (obj.GetType() != typeof(AccountDebited)) return false;
            return Equals((AccountDebited)obj);
        }

        public override int GetHashCode()
        {
            unchecked
            {
                int result = AggregateId.GetHashCode();
                result = (result*397) ^ Date.GetHashCode();
                result = (result*397) ^ (Payee != null ? Payee.GetHashCode() : 0);
                result = (result*397) ^ (Amount != null ? Amount.GetHashCode() : 0);
                result = (result*397) ^ (PaymentMethod != null ? PaymentMethod.GetHashCode() : 0);
                result = (result*397) ^ (Notes != null ? Notes.GetHashCode() : 0);
                result = (result*397) ^ (Categories != null ? Categories.GetHashCode() : 0);
                result = (result*397) ^ (Tags != null ? Tags.GetHashCode() : 0);
                return result;
            }
        }
    }
}