using System;
using Regalo.Core;

namespace Vita.Domain.Accounting
{
    public class AccountOpened : Event
    {
        public Guid        AggregateId       { get; private set; }
        public AccountType AccountType       { get; private set; }
        public Guid        AccountHolderId   { get; private set; }
        public string      AccountName       { get; private set; }
        public Money       OpeningBalance    { get; private set; }
        public string      AccountHolderName { get; private set; }

        public AccountOpened(Guid aggregateId, AccountType accountType, Guid accountHolderId, string accountHolderName, string accountName, Money openingBalance)
        {
            AggregateId       = aggregateId;
            AccountType       = accountType;
            AccountHolderId   = accountHolderId;
            AccountHolderName = accountHolderName;
            AccountName       = accountName;
            OpeningBalance    = openingBalance;
        }

        protected bool Equals(AccountOpened other)
        {
            return AccountHolderId.Equals(other.AccountHolderId) && string.Equals(AccountHolderName, other.AccountHolderName) && string.Equals(AccountName, other.AccountName) && AccountType == other.AccountType && AggregateId.Equals(other.AggregateId) && Equals(OpeningBalance, other.OpeningBalance);
        }

        public override bool Equals(object obj)
        {
            if (ReferenceEquals(null, obj)) return false;
            if (ReferenceEquals(this, obj)) return true;
            if (obj.GetType() != this.GetType()) return false;
            return Equals((AccountOpened)obj);
        }

        public override int GetHashCode()
        {
            unchecked
            {
                var hashCode = AccountHolderId.GetHashCode();
                hashCode = (hashCode * 397) ^ (AccountHolderName != null ? AccountHolderName.GetHashCode() : 0);
                hashCode = (hashCode * 397) ^ (AccountName != null ? AccountName.GetHashCode() : 0);
                hashCode = (hashCode * 397) ^ (int)AccountType;
                hashCode = (hashCode * 397) ^ AggregateId.GetHashCode();
                hashCode = (hashCode * 397) ^ (OpeningBalance != null ? OpeningBalance.GetHashCode() : 0);
                return hashCode;
            }
        }
    }
}