﻿using System;
using Regalo.Core;

namespace Vita.Domain.Accounting
{
    public class FundsTransfer : AggregateRoot
    {
        public void RequestFundsTransfer(Guid sourceAccountId, Guid destinationAccountId, Money amount)
        {
            if (sourceAccountId == destinationAccountId) throw new DomainException("You cannot transfer from an account to itself.");
            if (amount.Value <= 0) throw new DomainException("You cannot transfer zero or a negative amount.");

            Record(new FundsTransferRequested(Guid.NewGuid(), sourceAccountId, destinationAccountId, amount));
        }

        private void Apply(FundsTransferRequested evt)
        {
            Id = evt.FundsTransferId;
        }
    }
}