﻿using System;
using Regalo.Core;

namespace Vita.Domain.Accounting
{
    public class RequestFundsTransferHandler : ICommandHandler<RequestFundsTransfer>
    {
        private readonly IRepository<FundsTransfer> _fundsTransferRepository;
        private readonly IEventBus _eventBus;

        public RequestFundsTransferHandler(IRepository<FundsTransfer> fundsTransferRepository, IEventBus eventBus)
        {
            if (fundsTransferRepository == null) throw new ArgumentNullException("fundsTransferRepository");
            if (eventBus == null) throw new ArgumentNullException("eventBus");

            _fundsTransferRepository = fundsTransferRepository;
            _eventBus = eventBus;
        }

        public void Handle(RequestFundsTransfer command)
        {
            var fundsTransfer = new FundsTransfer();
            fundsTransfer.RequestFundsTransfer(command.SourceAccountId, command.DestinationAccountId, command.Amount);
            var events = fundsTransfer.GetUncommittedEvents();
            _fundsTransferRepository.Save(fundsTransfer);
            _eventBus.Publish(events);
        }
    }
}