using System;
using Dapper;
using Regalo.Core;
using Vita.Domain.Accounting;
using Vita.Infrastructure;

namespace Vita.SqlProjection.Accounting
{
    public class AccountOpenedHandler : IEventHandler<AccountOpened>
    {
        private readonly ISqlConnectionFactory _connectionFactory;

        public AccountOpenedHandler(ISqlConnectionFactory connectionFactory)
        {
            _connectionFactory = connectionFactory;
        }

        public void Handle(AccountOpened evt)
        {
            using (var connection = _connectionFactory.Create())
            {
                connection.Execute(
                    @"insert into Account (
                                Id, Name, AccountTypeName, AccountHolderId, AccountHolderName, CurrentBalance, Version
                            ) values (
                                @Id, @Name, @AccountTypeName, @AccountHolderId, @AccountHolderName, @CurrentBalance, @Version
                            )",
                    new
                    {
                        Id = evt.AggregateId.ToByteArray(), 
                        Name = evt.AccountName, 
                        AccountTypeName = evt.AccountType.ToString(), 
                        AccountHolderId = evt.AccountHolderId.ToByteArray(), 
                        AccountHolderName = evt.AccountHolderName, 
                        CurrentBalance = evt.OpeningBalance.Value, 
                        Version = Guid.NewGuid().ToByteArray()
                    });
            }
        }
    }
}