﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using Dapper;
using Regalo.Core;
using Vita.Domain.Accounting;
using Vita.Infrastructure;
using Vita.Web.Areas.Accounting.Models.Account;

namespace Vita.Web.Areas.Accounting.Controllers
{
    public class AccountController : Controller
    {
        private readonly ICommandProcessor _commandProcessor;
        private readonly ISqlConnectionFactory _sqlConnectionFactory;

        public AccountController(ICommandProcessor commandProcessor, ISqlConnectionFactory sqlConnectionFactory)
        {
            _commandProcessor = commandProcessor;
            _sqlConnectionFactory = sqlConnectionFactory;
        }

        [HttpGet]
        public ActionResult Open()
        {
            ViewBag.AccountTypes = Enum.GetValues(typeof(AccountType)).Cast<AccountType>();
            return View();
        }

        [HttpPost]
        public ActionResult Open(OpenAccountViewModel model)
        {
            _commandProcessor.Process(new OpenAccount(MvcApplication.UserId, model.AccountName, model.AccountType, new Money(model.OpeningBalanceAmount)));
            return RedirectToAction("Index", "Home", new {area = "Accounting"});
        }

        [HttpGet]
        public ActionResult RecentTransactions(Guid id)
        {
            AccountViewModel account;
            IEnumerable<AccountTransactionViewModel> transactions;
            using (var connection = _sqlConnectionFactory.Create())
            {
                account = connection.Query<AccountViewModel>(
                    "SELECT Name, CurrentBalance FROM Account WHERE Id = @id",
                    new { id = id.ToByteArray() })
                                    .SingleOrDefault();
                transactions = connection.Query<AccountTransactionViewModel>(
                    "SELECT Date, Amount, Balance, Description, Payee, PaymentMethod FROM Transaction WHERE AccountId = @accountId ORDER BY Date DESC",
                    new { accountId = id.ToByteArray() });
            }

            return View(new RecentTransactionsViewModel {Account = account, Transactions = transactions});
        }
    }
}
