﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using Vita.Domain.Accounting;
using Vita.Infrastructure;
using Vita.Web.Areas.Accounting.Models.Home;
using Dapper;

namespace Vita.Web.Areas.Accounting.Controllers
{
    public class HomeController : Controller
    {
        private readonly ISqlConnectionFactory _connectionFactory;

        public HomeController(ISqlConnectionFactory connectionFactory)
        {
            _connectionFactory = connectionFactory;
        }

        public ActionResult Index()
        {
            var model = new IndexViewModel();

            using (var connection = _connectionFactory.Create())
            {
                model.Accounts = connection.Query("select * from Account /*where AccountHolderId = @AccountHolderId*/ order by Name", new {AccountHolderId = MvcApplication.UserId})
                    .Select(row => new AccountListItemViewModel
                                       {
                                           Id = new Guid(row.Id),
                                           Version = new Guid(row.Version),
                                           Name = row.Name,
                                           CurrentBalance = new Money(row.CurrentBalance),
                                           AccountHolderName = ""
                                       });
            }

            return View(model);
        }
    }
}
