﻿using System;
using System.IO;
using System.Reflection;
using Castle.MicroKernel.Registration;
using Castle.MicroKernel.SubSystems.Configuration;
using Castle.Windsor;
using Regalo.Core;

namespace Vita.Web.Infrastructure
{
    public class CommandHandlerInstaller : IWindsorInstaller
    {
        public void Install(IWindsorContainer container, IConfigurationStore store)
        {
            container.Register(
                Component
                    .For<ICommandProcessor>()
                    .ImplementedBy<CommandProcessor>()
                    .LifestylePerWebRequest());

            container.Register(
                Component
                    .For<IEventBus>()
                    .ImplementedBy<EventBus>()
                    .LifestylePerWebRequest());

            container.Register(
                AllTypes
                    .FromAssemblyInDirectory(new AssemblyFilter(AssemblyDirectory, "Vita.*.dll"))
                    .BasedOn(typeof(ICommandHandler<>))
                    .WithServiceFirstInterface()
                    .LifestylePerWebRequest());

            container.Register(
                AllTypes
                    .FromAssemblyInDirectory(new AssemblyFilter(AssemblyDirectory, "Vita.*.dll"))
                    .BasedOn(typeof(IEventHandler<>))
                    .WithServiceFirstInterface()
                    .LifestylePerWebRequest());
        }

        static public string AssemblyDirectory
        {
            get
            {
                var codeBase = Assembly.GetExecutingAssembly().CodeBase;

                var uri = new UriBuilder(codeBase);

                var path = Uri.UnescapeDataString(uri.Path);

                return Path.GetDirectoryName(path);
            }
        }
    }
}