﻿using Castle.MicroKernel.Registration;
using Castle.MicroKernel.SubSystems.Configuration;
using Castle.Windsor;
using Raven.Client;
using Raven.Client.Document;
using Regalo.Core;
using Regalo.Core.EventSourcing;
using Regalo.RavenDB;

namespace Vita.Web.Infrastructure
{
    public class RepositoryInstaller : IWindsorInstaller
    {
        public void Install(IWindsorContainer container, IConfigurationStore store)
        {
            var documentStore = new DocumentStore { ConnectionStringName = "EventStore"};
            documentStore.Initialize();

            container.Register(
                Component.For<IDocumentStore>()
                    .Instance(documentStore),
                Component.For(typeof(IEventStore))
                    .ImplementedBy(typeof(RavenEventStore))
                    .LifestylePerWebRequest(),
                Component.For<IConcurrencyMonitor>()
                    .ImplementedBy<StrictConcurrencyMonitor>()
                    .LifestylePerWebRequest(),
                Component.For(typeof(IRepository<>))
                    .ImplementedBy(typeof(EventSourcingRepository<>))
                    .LifestylePerWebRequest(),
                Component.For<IVersionHandler>()
                    .ImplementedBy<VitaVersionHandler>()
                    .LifestylePerWebRequest()
                );
        }
    }
}